#include <fstream.h>
#include <math.h>

/*  Copyright  1997. All Rights Reserved.

	FILE: 		daubwave.cpp
	AUTHOR:     Przemysaw liwiski
	e-mail:		sliwka@diuna.ict.pwr.wroc.pl

	OVERVIEW
	========
	Source file for implementation of wavelet recursive calculation. According
	to G.Strang SIAM Review (31) No. 4 pp 614-627 December 1989
	"Wavelets and Dilatation Equations: A Brief Introduction"
                                                           
	P remark to above paper:
	1.	At page 619 the chart of W4 is incorrect, it illustrates -W4 indeed.
                                                    
	///... method descriptions:

	'DW::InitCoeffs(const ScaleFunction)' initializes filter coefficients

	'DW::DSF(double x, const uint level)' recursive method calculating value of
										  Daubechies' scaling function @ point x
										  with given 'level' accuracy

	'DW::DSW(double x, const uint level)' recursive method calculating value of
										  Daubechies' wavelet function @ point x
										  with given 'level' accuracy

	'DW::HaarFunction(const double x)'    returns '1' if(x>=0 & x<1) and '0'
										  otherwise (implemented as 'inline')
*/


#include "Daub4.h"

double DW::c12[12]=
{  0.111540743350, 0.494623890398, 0.751133908021,
   0.315250351709,-0.226264693965,-0.129766867567,
   0.097501605587, 0.027522865530,-0.031582039318,
   0.000553842201, 0.004777257511,-0.001077301085
};

double DW::c20[20]=
{  0.026670057901, 0.188176800078, 0.527201188932,
   0.688459039454, 0.281172343661,-0.249846424327,
  -0.195946274377, 0.127369340336, 0.093057364604,
  -0.071394147166,-0.029457536822, 0.033212674059,
   0.003606553567,-0.010733175483, 0.001395351747,
   0.001992405295,-0.000685856695,-0.000116466855,
   0.000093588670,-0.000013264203
};

DW::DW(const uint type)
   :initialized(1)
{
	InitCoeffs(ScaleFunction(type));
}

void DW::InitCoeffs(const DW::ScaleFunction t=DW::D4)
{
	type=t;

	int i;
	double	sqrt3=sqrt(3);
	double	sqrt10=sqrt(10);
	double  sqrt5210=sqrt(5+2*sqrt10);
	switch(type)
	{
		case Haar:  C[0]=C[1]=1.0;
					C[2]=C[3]=0.0;
		break;
		case Hat:  	C[0]=C[2]=0.5;
					C[1]=1.0;
					C[3]=0.0;
		break;
		case D4:	C[0]=0.25*(1.0+sqrt3);
					C[1]=0.25*(3.0+sqrt3);
					C[2]=0.25*(3.0-sqrt3);
					C[3]=0.25*(1.0-sqrt3);
		break;
		case D6:
					C[0]=(1 +  sqrt10+  sqrt5210)*0.0625;
					C[1]=(5 +  sqrt10+3*sqrt5210)*0.0625;
					C[2]=(10-2*sqrt10+2*sqrt5210)*0.0625;
					C[3]=(10-2*sqrt10-2*sqrt5210)*0.0625;
					C[4]=(5 +  sqrt10-3*sqrt5210)*0.0625;
					C[5]=(1 +  sqrt10-  sqrt5210)*0.0625;
		break;
		///...	for longer filters I had only tabularized, numerical coeffs
		case D12:	for(i=0;i<D12;i++)
					{
						C[i]=c12[i];
					}
		break;
		case D20:	for(i=0;i<D20;i++)
					{
						C[i]=c20[i];
					}
		break;
		default:	for(i=0;i<20;i++)
					{
						C[i]=0.0;
					}
		break;
	}
}

double DW::DSF(double value,uint level)
{
	double 	temp=2*value;
	double 	fi=0.0;
	if(level)
	{
		level--;
		for(int k=0;k<type;k++)
		{
			fi+=C[k]*DSF(temp-k,level);
		}
		return fi;
	}
	for(int k=0;k<type;k++)
	{
		fi+=C[k]*HaarFunction(temp-k);
	}
	return fi;
/*	double temp=2*value;
	double fi;
	if(level)
	{
		level--;
		fi=C[0]*DSF(temp  ,level)+C[1]*DSF(temp-1,level)
		  +C[2]*DSF(temp-2,level)+C[3]*DSF(temp-3,level);
		return fi;
	}
	fi=C[0]*HaarFunction(temp  )+C[1]*HaarFunction(temp-1)
	  +C[2]*HaarFunction(temp-2)+C[3]*HaarFunction(temp-3);
	return fi;
*/}

double DW::DSW(double value,uint level)
{
/*	double w;
	double temp=2*value;
	w=-C[0]*DSF(temp-1,level)+C[1]*DSF(temp  ,level)
	  -C[2]*DSF(temp+1,level)+C[3]*DSF(temp+2,level);
	return w;
	//...	implementation of Daubechies' wavelet with more than 4 coeffs
	//...	... only for absolutely patient people !!!
*/	double w=0.0;
	double temp=2*value;
	//...	k runs from (-1) to (+2) but C's coefficients run as (k+1)

	for(int k=-1;k<type-1;k++)
	{
		w+=(k & 0x01L ? -1 : 1)*C[k+1]*DSF(temp+k,level);
	}
	return w;
}

int DW::SupportInf(DW::FunctionType type,uint number)
{
    int Kmax=number-1;
    int Kmin=0;
    int N=(Kmax-Kmin+1)/2;

    return type ? 1-N : Kmin;
}



int DW::SupportSup(DW::FunctionType type,uint number)
{
    int Kmax=number-1;
    int Kmin=0;
    int N=(Kmax-Kmin+1)/2;

    return type ? N : Kmax;
}

int main()
{
	double	resolution=0.0;
	double 	min,max,range;
	int 	percent,oldPercent;
	int   	recursionMin,recursionMax;
	int 	type;

	cout<<"1997  SLK\nDaubechies, Haar & hat mother & father wavelet example";
	cout<<"\n\n0-Hat 2-Haar n-Daubechies (n==2(Haar),4,6,12,20)";
	cout<<"\nSelect the father wavelet type : ";
	cin>>type;
	cout<<"\nInput the recursion range\nfrom : ";
	cin>>recursionMin;
	cout<<"\nto   : ";
	cin>>recursionMax;

    DW      dw(type);
	if(dw.Type()==DW::Hat)
	{
		min=-0.5;
		max=2.0;
	}
	else
	{
		min=1-dw.Type()/2.0;
		max=dw.Type()-1.0;
	}

    range=max-min;
	cout<<"\nThe support is : "<<range;
	cout<<"\nSo, finally input the number of points (0 - default 2048): ";
	cin>>resolution;

	resolution=fabs(resolution==0.0 ? 2048 : resolution);
	cout<<"\nWavelet function at "<<resolution<<" points will be computed: ";
	ofstream ofs;
    char filename[]="DB-xstep.dat";
    for(int i=recursionMin;i<recursionMax+1;i++)
    {
        filename[3]=char(i+'0');
    	cout<<"\nWriting to file \'"<<filename<<"\'...\n";
        ofs.open(filename);
		oldPercent=0;
        for(double x=min;x<=max;x+=(1/resolution)*range)
        {
            percent=(x-min)*100/(max-min);
            if(percent!=oldPercent)
            {
            	cout<<percent<<"%";
	            cout<<"\b\b\b\b";
                oldPercent=percent;
            }
            ofs.precision(14);
            ofs<<x;
            ofs.precision(14);
            ofs<<"\t"<<dw.DSF(x,i);
            ofs.precision(14);
            ofs<<"\t"<<dw.DSW(x,i)<<endl;
        }
        ofs.close();
        cout<<percent<<"%\nOK!\n";
    }
	return 0;
}
